part of plot_window;

class LinePlotWindow extends PlotWindow {

  List<Line> lines = new List<Line>();

  Range _fixedRange;
  
  Range get fixedRange => _fixedRange ??= new Range( lines.map((Line l)=>l.smoothedPointList.first.x).reduce(math.min),
      lines.map((Line l)=>l.smoothedPointList.last.x).reduce(math.max));

  Range get croppedRange => new Range(rectangle.left, rectangle.right);
  
  NumSetting get startOfLinearRegressionSetting => new NumSetting("set start of linear regression", this.fixedRange.start, (num n) { this.fixedRange.start = n; });
  NumSetting get endOfLinearRegressionSetting => new NumSetting("set end of linear regression", this.fixedRange.end, (num n) { this.fixedRange.end = n; });
  
  
  LinePlotWindow(CanvasElement canvas):super(canvas);
  
  Iterable<Performer> getPerformers()=>[startOfLinearRegressionSetting, endOfLinearRegressionSetting];
  
  void addLine(Line line) {
    if (line.pointList == null || line.validPointList.points.isEmpty || lines.contains(line)) return;
    line.color??=colorIterator.getNext();
    lines.add(line);
  }
  
  void removeData() {
    this.lines.clear();
  }
  
  void removeLine(String s) {
    this.lines.remove(s);
  }
  
  Rectangle getInitialSize()=>lines.map((Line l)=>l.rectangle).reduce((value, element)=>value.boundingBox(element));
  
  void drawContent() {
    for (Line line in lines) {
      var points = line.smoothedPointList.points;
      points = filterToRectangle(points);
      if (points.isNotEmpty) {
        if (line.lineType.hasLines) {
          context.save();
          context.setStrokeColorRgb(line.color.red, line.color.green, line.color.blue);
          context.lineWidth = line.width;
          Point first = toCanvas(points[0]);
          context..beginPath()
            ..moveTo(first.x, first.y);
          points.skip(1).map(toCanvas).forEach((Point p)=>context.lineTo(p.x, p.y));
          context.stroke();
          context.restore();
        }
        if (line.lineType.hasPoints) {
          context.save();
          context.fillStyle = "${line.color}";
          points.map(toCanvas).forEach((Point p) {
            context..moveTo(p.x, p.y)
              ..beginPath()
            ..arc(p.x, p.y, line.pointRadius, 0, 2*math.PI)
              ..fill();
          });
          context.restore();
        }
      }
      if (line.showLinearRegression) {
        line.range = line.pickRange(fixedRange, croppedRange);
        var linearRegression = line.linearRegression;

        Point first = toCanvas(new Point(rectangle.left, linearRegression.f(rectangle.left)));
        context.save();
        context.setStrokeColorRgb(line.color.red, line.color.green, line.color.blue);
        context.lineWidth = line.width;
        context.setLineDash([5]);
        context..beginPath()
          ..moveTo(first.x, first.y);
        Point last = toCanvas(new Point(rectangle.right, linearRegression.f(rectangle.right)));
        context.lineTo(last.x, last.y);
        context.stroke();
        context.restore();
      }
    }
    blankOutBorders();
  }

  Iterable<LegendEntry> getLegendEntries() {
    List<LegendEntry> legendEntries = new List<LegendEntry>();
    for(Line line in lines) {
      List<String> sublist = new List<String>()..add(line.name);
      if(line.showLinearRegression) {
        line.range = line.pickRange(fixedRange, croppedRange);
        sublist.add("${line.linearRegression}"); //smoothedPointList.getLinearRegressionString(line.linearRegressionFormat));
      }
      legendEntries.add(new LegendEntry(line.color, sublist, line));
    }
    return legendEntries;
  }

}