part of plot_window;

typedef num PointMapper(Point p);
typedef num NumReducer(num n1, num n2);

class LinearRegression {

  static NumReducer _ADDER = (num n1, num n2)=>n1+n2;

  static num _ReducePoints(List<Point> points, PointMapper pm, [NumReducer nr = null]) {
    if (nr==null) nr = _ADDER;
    if (points.length==0) return null;
    return points.map(pm).reduce(nr);
  }

  List<Point> points;

  num f(num x)=>x*slope + offset;
  
  String toString() {
    if (slope==null || offset==null || slope.isNaN || offset.isNaN) return "undefined";
    var nf = new intl.NumberFormat(format);
    var mString = nf.format(slope);
    var cString = nf.format(offset);
    var eString = nf.format(error);
    if (offset<0) return "${mString}x ${cString} (±$eString)";
    return "${mString}x +$cString (±$eString)";
  }

  num _slope;

  num get slope {
    if (_slope==null) { _setLinearRegression(); }
    return _slope;
  }
    
  num _offset;

  num get offset {
    if (_offset==null) { _setLinearRegression(); }
    return _offset;
  }
  
  num _error;

  num get error => _error ??= _ReducePoints(points, (Point p)=>(p.x * slope + offset - p.y).abs())/points.length;

  num __sumY;
  num get _sumY => __sumY ??= _ReducePoints(points, (Point p)=>p.y);
  num get average => _sumY/points.length;

    /*
    y = m*x + c
    zu minimieren ist das Quadrat der Abstaende:
    s = sum( (y_i - (m*x_i +c))^2) 
    ds/dm = 2 * sum ( (y_i - (m*x_i + c)) * -x_i = 0
    S (x*y - m*x^2 - c*x ) = 0
    m * S (x^2) + c * S (x) = S (x*y)
    und

    ds/dc = 2 * sum( y - (m*x + c)) = 0
    S ( y - m*x - c) = 0
    m * S (x) + c * S(1) = S(y)

    | S(x^2)  S(x) |   | m |   | S(x*y) |
    |              | x |   | = |        |
    | S(x)    S(1) |   | c |   | S(y)   |

    AX = B
    X = A^{-1}B
   */

  
  void _setLinearRegression() {
    if (points.isEmpty) return;
    num sumX2 = _ReducePoints(points, (Point p)=>math.pow(p.x, 2));
    num sumX = _ReducePoints(points, (Point p)=>p.x);
    num sum = points.length;
    num sumXY = _ReducePoints(points, (Point p)=>p.x*p.y);
    num sumY = _sumY;
    num det = sum*sumX2 - sumX*sumX;
    _slope = (sum * sumXY - sumX*sumY)/det;
    _offset = (-sumX*sumXY + sumX2*sumY) / det;
  }

  String format;

  Range getRange() => new Range(this.points.first.x, this.points.last.x);

  LinearRegression(this.points, this.format);

}